//This reference code is for the MSP430F2619. The code will stay same for
//all controllers with only difference of header files and controller based
// port, register and clock settings

//#define TAGNUMBER 5 // Set the Tag Number for the corresponding frequency
//#define __MSP430F2619__

#include "msp430x26x.h"
#include <stdlib.h>
#include <settings.h>
//#include <siggen.h>
#include <spi.h>

struct Sfreq	// Structure to hold PLL counter values for a frequency
{
	int iFreq;
	int iA;
	int iB;
};

__interrupt void timerA0_isr(void);  // TimerA0 isr


/**
 * Initializes MSP430 peripherals and clock.
 */
void init_sys(void)
{
  /* Allow watchdog timer to run */
  WDTCTL = WDTPW | WDTHOLD;					// Stop Watchdog timer	
	
  //SVSCTL=0xB0|PORON;			// Power on at 3.2V
  //while(SVSCTL&SVSFG){
  //  SVSCTL=SVSCTL&(!SVSFG);
  //}
  
  DCOCTL = CAL_DCO_1MHZ;		// Set DCO for calibrated 1MHz
  BCSCTL1 = CAL_BC1_1MHZ;
  
  
// Configure all unused port pins as outputs.
  P1DIR = ~(BIT0|BIT1);                             // Set port to outputs
  P1OUT = 0;                                // Reset port outputs
  P1SEL = 0;
   
  P2DIR = 0xFF;                             // Set port to outputs
  P2OUT = 0; 								// Reset port outputs
  P2SEL = 0;
  
  P3DIR = 0xff;                             // Set port to outputs
  P3OUT = 0;                                // Reset port outputs
  P3SEL = BIT0|BIT4|BIT5;					// UCA0 
  
 /* P4DIR = ~(BIT6);                             // Set port to outputs
  P4OUT = 0;                                // Reset port outputs
  P4SEL = 0;
  */
  
  P4DIR = 0xFF; //~BIT6;                             // Set port to outputs
  P4OUT = BIT6;                                // Reset port outputs ; pull HMC900EN high
  P4SEL = 0;
  //P4REN = BIT6;
  
  /*P5DIR = ~(BIT0|BIT1|BIT2|BIT3|BIT5|BIT6);                             // Set port to outputs
  P5OUT = 0;                                // Reset port outputs
  P5SEL = 0; *///BIT4;								// Output MCLK on pin P5.4
  
  P5DIR = ~BIT0; //~(BIT0|BIT6);                             // Set port to outputs
  P5OUT = BIT0|BIT6;                                // Reset port outputs ; Pull HMC960EN high ; pull UCB1STE high
  P5SEL = BIT3|BIT2|BIT1; //BIT4;								// Output MCLK on pin P5.4
  P5REN = BIT0; //|BIT6;
  
  P6DIR = 0xFF;                             // set unused pins to outputs
  P6OUT = 0;								// Reset port outputs
  P6SEL = BIT0|BIT1|BIT3|BIT4|BIT5;
  
 
// Configure UCA0 for PLL 
  UCA0CTL1 = UCSWRST;						// Disable UCA0
  UCA0CTL0 = UCCKPH | UCMSB | UCMST | UCSYNC;	// Set UCA0 for MSB and master modes
  UCA0CTL1 = UCSSEL_2 | UCSWRST;			// Source UCA0 from SMCLK at 16MHz
  UCA0CTL1 &= ~UCSWRST;			// Enable UCA0

// Configure UCB1 for HMC900, HMC960, and FPGA 
  UCB1CTL1 = UCSWRST;						// Disable UCB1
  UCB1CTL0 = UCCKPH | UCMSB | UCMST | UCSYNC;	// Set UCB1 for MSB and master modes
  UCB1CTL1 = UCSSEL_2 | UCSWRST;			// Source UCB1 from SMCLK at 1MHz
  UCB1BR0 = 0;
  UCB1CTL1 &= ~UCSWRST;			// Enable UCA0

  
// Configure TIMER A0 for PLL lock and watchdog
  TACCTL0 = CCIE;
  TACCR0 = 10000;
  TACTL = TASSEL_2 | MC_1;
  
} 

/**
 * Returns a structure of PLL counter values corresponding
 * to a given frequency.
 * 
 * @param iFreq Desired frequency.
 * @return Sfreq structure containing PLL counter values.
 */
 struct Sfreq sFrequency(int iFreq)
 {
 	struct Sfreq sFreqTemp;
 	int iP;
 	
  	if(iFreq < 5725) sFreqTemp.iFreq = 5725;
  	else if(iFreq > 5875) sFreqTemp.iFreq = 5875;
  	else sFreqTemp.iFreq = iFreq;
  
  	iP = 64;
  	sFreqTemp.iB = 88;
  	sFreqTemp.iA = 0;
  	do{
    	sFreqTemp.iB++;
    	sFreqTemp.iA = sFreqTemp.iFreq - (iP * sFreqTemp.iB);
  	}while((sFreqTemp.iA < 0) || (sFreqTemp.iA > 63));
  
  	return sFreqTemp;
 }
 
/**
 * Transmits data over SPI to PLL
 * 
 * @param ulData Data to be transmitted to PLL.
 */
void vTransmit(unsigned long ulData){
  char cTmp;
  int i;
  
  i = 3;
  
  P3OUT = P3OUT & ~BIT3;  		// Pull LE low
  do{
    i--;
    
    cTmp = (ulData >> (8 * i)) & 0xFF;
    
    UCA0TXBUF = cTmp; 
    
    while(!(IFG2 & UCA0TXIFG));
   
  }while(i != 0);
   
  P3OUT = P3OUT | BIT3;     // Pull LE high to latch data.
} 
 
 
/**
 * Output frequency counter values to PLL via SPI
 * 
 * @param sFreq Frequency structure containing PLL counter values.
 */
void vPLL_Transmit(struct Sfreq sFreq)
{
	unsigned long ulBuffer;
	
	ulBuffer = 0xDF809A;
	vTransmit(ulBuffer);
	  
	ulBuffer = 0x000040;
	vTransmit(ulBuffer);
	
	ulBuffer = (sFreq.iB << 8) + (sFreq.iA << 2) + 1;
	vTransmit(ulBuffer);
	  
	ulBuffer = 0xDF8092;
	vTransmit(ulBuffer); 
}


/**
 * Main entry point.
 * 
 */

void main()
{
	int iFreq,addr;
	char rxBuf;
	unsigned long data;
	init_sys();								// Initialize processor
	
	//iFreq = 5710 + (TAGNUMBER * 20);		// Calculate frequency of tag 5730,5750..5870
	iFreq = 5800;	
	//vPLL_Transmit(sFrequency(iFreq));		// Set PLL frequency to 5800MHz
	
     //   LPM3;
        
        __delay_cycles(2550);
        __delay_cycles(2550);
        __delay_cycles(550);
 
        
       
	while(1){
		__bis_SR_register(LPM0_bits+GIE);  	// Go into LPM3 with interrupts enabled
		
		// Check PLL lock 
		rxBuf = UCA0RXBUF;					// Clear UCA0 rx buffer
		UCA0TXBUF = 0;						// Send dummy data to PLL
		while(!(IFG2 & UCA0RXIFG));			// Wait for recieved data
		rxBuf = UCA0RXBUF;					// Read muxout
		
		if( rxBuf != 0xFF )					// If not locked...
		{
			vPLL_Transmit(sFrequency(iFreq));  // Reprogram PLL
		}
	}
        
}

/**
 * Timer A0 interrupt service
 * 
 * Reset watchdog and let main routine check pll lock
 */
#pragma vector=TIMERA0_VECTOR
__interrupt void timerA0_isr(void)
{
	// Clear watchdog
	WDTCTL = WDTPW | WDTCNTCL;
	
	//P5OUT ^= BIT5;							// Toggle pin 5.5 for debug
	
	__bic_SR_register_on_exit(LPM0_bits);  // Exit LPM3
}




int spiTranceive(char *dest,char *src,int size,char dev)
{
	volatile unsigned char *pTXbuf;
       // unsigned char *pRXbuf;
	unsigned char const volatile *pRXbuf;
	volatile unsigned char *pIFGreg;
	char cIFGtx;
	char cIFGrx;
	volatile unsigned char *pSlavePort;
	volatile unsigned char *pSlaveDir;
	volatile unsigned char *pSlaveRen;
	char cSlaveBit;
	int i;
		
	switch(dev)
	{
		case PLL:
		case PLL_LOCKDET:
		case UCA0:
			pTXbuf = &UCA0TXBUF;
			pRXbuf = &UCA0RXBUF;
			pIFGreg = &IFG2;
			cIFGtx = UCA0TXIFG;
			cIFGrx = UCA0RXIFG;
		break;
		
		case UCA1:
			pTXbuf = &UCA1TXBUF;
			pRXbuf = &UCA1RXBUF;
			pIFGreg = &UC1IFG;
			cIFGtx = UCA1TXIFG;
			cIFGrx = UCA1RXIFG;
		break;
		
		case UCB0:
			pTXbuf = &UCB0TXBUF;
			pRXbuf = &UCB0RXBUF;
			pIFGreg = &IFG2;
			cIFGtx = UCB0TXIFG;
			cIFGrx = UCB0RXIFG;
		break;
		
		case HMC900:
		case HMC960:
		case FPGA:
		case UCB1:
			pTXbuf = &UCB1TXBUF;
			pRXbuf = &UCB1RXBUF;
			pIFGreg = &UC1IFG;
			cIFGtx = UCB1TXIFG;
			cIFGrx = UCB1RXIFG;
		break;
		
		default:
			pTXbuf = NULL;
			pRXbuf = NULL;
			pIFGreg = NULL;
			cIFGtx = 0;
			cIFGrx = 0;
			size = 0;
		break;
	}
	
	switch(dev)
	{
		case HMC900:
			pSlavePort = &P4OUT;
			pSlaveDir = NULL; //&P4DIR;
			pSlaveRen = NULL; //&P4REN;
			cSlaveBit = BIT6;
		break;
		
		case HMC960:
			pSlavePort = &P5OUT;
			pSlaveDir = NULL; //&P5DIR;
			pSlaveRen = NULL; //&P5REN;
			cSlaveBit = BIT6;
		break;
		
		case UCA0:
		case PLL:
			pSlavePort = &P3OUT;
			pSlaveDir = &P3DIR;
			pSlaveRen = &P3REN;
			cSlaveBit = BIT3;
		break;
		
		case PLL_LOCKDET:
			pSlavePort = NULL;
			pSlaveDir = NULL;
			pSlaveRen = NULL;
			cSlaveBit = 0;
		break;
		
		case UCB1:
		case FPGA:
			pSlavePort = &P5OUT;
			pSlaveDir = &P5DIR;
			pSlaveRen = &P5REN;
			cSlaveBit = BIT0;
		break;
		
		case UCB0:
			pSlavePort = &P3OUT;
			pSlaveDir = &P3DIR;
			pSlaveRen = &P3REN;
			cSlaveBit = BIT0;
		break;
		
		case UCA1:
			pSlavePort = &P5OUT;
			pSlaveDir = &P5DIR;
			pSlaveRen = &P5REN;
			cSlaveBit = BIT3;
		break;
		
		default:
			pSlavePort = NULL;
			pSlaveDir = NULL;
			pSlaveRen = NULL;
			cSlaveBit = 0;
			size = 0;
		break;
	}
	
	*pRXbuf;					// Clear receive buffer
	
	*pSlaveDir |= cSlaveBit;	// Set enable to output
	*pSlaveRen &= ~cSlaveBit;	// Disable pull up resistor
	*pSlavePort &= ~cSlaveBit;	// Pull enable low
	
	i = size;
	do{
		i--;
		while(!(*pIFGreg & cIFGtx));	// Wait for transmit buffer to clear
		*pTXbuf = src[i];				// Send byte
		while(!(*pIFGreg & cIFGrx));	// Wait for received byte
		
		if(dest != NULL) dest[i] = *pRXbuf;		// Write received byte to dest
		else *pRXbuf;							// Clear rx buffer
		
	}while(i > 0);
	
	*pSlavePort |= cSlaveBit;	// Pull enable high
	*pSlaveRen	|= cSlaveBit;	// Enable pull up resistor
	*pSlaveDir &= ~cSlaveBit;	// Set enable to input	
	
	return (size);	
}